/*
 * Copyright (C) 2006, 2007, 2009 Toni Corvera
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// $Id: str_utils.h 1590 2009-04-05 17:33:40Z  $

#ifndef ONET_STR_UTILS_H
#define ONET_STR_UTILS_H

/*!
 * \file
 * \remarks This is a subset of a bigger str module I used on other programs
 * \remarks Error handling is not always graceful either
 */

#include <string>
#include <sstream>
#include <vector>
#include <cassert>
#include <iomanip> // setfill, setw

namespace net_outlyer {

/*!
 * \brief This namespace contains miscellaneous string utilities
 * \todo TODO: Optimise where appropiate
 */
namespace str {

/*!
 * \brief alias for std::ios_base callbacks
 * \remarks base_t is the same as std::ios_base & (*fn)(std::ios_base &)
 * \see to_string(const T&, base_t)
 */
typedef std::ios_base & (base_t)(std::ios_base &);

/*!
 * \brief Converts a given value to a string
 * \param[in] v Value to convert
 * \param[in] base Numeric base to use. Only meaningful for scalars.
 * \param[in] minwidth minimal width
 * By default the decimal base is used.
 * \note v should have a string conversion operator
 * \see http://www.codeguru.com/forum/showthread.php?t=231056 for the original
 * \remarks T should be able to print to a stream
 * \see to_hex(int)
 */
template<typename T> std::string to_string(const T & v,
                                           base_t base = std::dec,
                                           int minwidth = 0) {
    std::ostringstream ss;
    ss << std::setfill('0') << std::setw(minwidth) << base << v;
    return ss.str();
}

//! Shorthand version of to_string(const T&, base_t)
inline std::string to_string(int n) { return to_string<int>(n); }
//! Shorthand version of to_string(const T&, base_t)
inline std::string to_string(unsigned int n) { return to_string<unsigned int>(n); }

/*!
 * \brief Returns the hex representation of a scalar number
 * \see to_string(const T&, base_t)
 */
std::string to_hex(int);

/*!
 * \brief Like to_hex(int) but pretty printing
 * \param minbytes Minimum number of bytes to print (zeros will be added to
 *                 pad) note that 1 means *two* characters
 * \param zeroex Prepend 0x
 */
std::string to_hex(int, unsigned int minbytes, bool zeroex = false);

/*!
 * \brief Returns an uppercase copy of the string
 * \param[in] s Input string
 * \return A new string, same as input string but with uppercase characters
 */
std::string to_upper(const std::string & s);

/*!
 * \brief Returns a lowercase copy of the string
 */
std::string to_lower(const std::string &);

/*!
 * \brief Defines the direction of the padding
 * \see pad()
 */
enum pad_direction {
    PAD_LEFT, /*!< \brief Add padding on the left */
    PAD_RIGHT,    /*!< \brief Add padding on the right */

    /*!
     * \brief Add padding by both sides
     * \note When using \em PAD_BOTH, if padding is odd, the right padding
     *       will be bigger than the right padding
     */
    PAD_BOTH
};

/*!
 * \brief Returns a copy of s padded with padding
 * \param[in] s Input string
 * \param[in] pad_to Minimum number of characters
 * \param[in] padding String to use as padding
 * \param[in] pad_dir Direction of the padding (where will the padding be
 * added)
 * \return A new string with the padded input string
 */
std::string pad(const std::string & s,
                size_t pad_to,
                const std::string & padding = " ",
                const pad_direction & pad_dir = PAD_RIGHT);

/*!
 * \brief Returns a copy of s padded with padding
 * \param[in] s Input string
 * \param[in] pad_to Minimum number of characters
 * \param[in] padding character to use as padding
 * \param[in] pad_dir Direction of the padding (where will the padding be
 * added)
 * \return A new string with the padded input string
 */
std::string pad(const std::string & s,
                size_t pad_to,
                char padding,
                const pad_direction & pad_dir = PAD_RIGHT);

/*!
 * \brief Checks if a string's ending matches a second given string
 * \param[in] s string to check
 * \param[in] end ending to look for
 * \return true if \em s ends in \em ending
 */
bool ends_in(const std::string & s, const std::string & end);

} // namespace str

} // namespace net_outlyer

#endif // ONET_STR_UTILS_H
// vim:set ts=4 et ai:
